#include "opencv2/opencv.hpp"
#include "opencv2/ximgproc.hpp"

void seed_and_grow(cv::Mat &img, cv::Mat &labels)
{
    int num_superpixels = 800;
    int num_iterations = 10;
    int prior = 2;
    int num_levels = 4;
    int num_histogram_bins = 5;

    cv::Mat converted_img;
    cv::cvtColor(img, converted_img, cv::COLOR_BGR2HSV);
    int height = converted_img.rows;
    int width = converted_img.cols;
    int channels = converted_img.channels();

    cv::Ptr<cv::ximgproc::SuperpixelSEEDS> seeds = cv::ximgproc::createSuperpixelSEEDS(width, height, channels, num_superpixels, num_levels, prior, num_histogram_bins);

    seeds->iterate(converted_img, num_iterations);

    //retrieve the segmentation result
    seeds->getLabels(labels);

    cv::Mat mask, mask_inv, result_bg, result_fg, result;
    seeds->getLabelContourMask(mask, false);

    //stitch foreground & background together
    cv::bitwise_not(mask, mask_inv);
    cv::bitwise_and(img, img, result_bg, mask_inv);
    cv::Mat color_img(height, width, CV_8UC3, cv::Scalar(0, 0, 255));
    cv::bitwise_and(color_img, color_img, result_fg, mask);
    cv::add(result_bg, result_fg, result);

    cv::imshow("result", result);
    cv::imshow("mask", mask);
    cv::waitKey(1);
}

void graph_segmentation(cv::Mat &img, cv::Mat &labels)
{
    double sigma = 0.5; // 0.8
    float k = 300;
    int min_size = 100;
    cv::Ptr<cv::ximgproc::segmentation::GraphSegmentation> gs = cv::ximgproc::segmentation::createGraphSegmentation(sigma, k, min_size);
    gs->processImage(img, labels);
    cv::imshow("image", img);
    cv::waitKey(1);
}

int main()
{
    int segmentation_type;
    std::cout << "enter 1 (seed & grow) or 2 (graph segmentation): ";
    std::cin >> segmentation_type;
    cv::Mat img = cv::imread("../lena.jpg", cv::IMREAD_COLOR), labels;

    if (segmentation_type == 1) { // region seed & grow
        seed_and_grow(img, labels);
    }
    else if (segmentation_type == 2) { // graph segmentation
        graph_segmentation(img, labels);
    }

    int label_count = *std::max_element(labels.begin<int>(), labels.end<int>()) + 1;
    std::vector<cv::Vec3i> colors(label_count, 0);
    std::vector<int> counts(label_count, 0);
    for (int y = 0; y < labels.rows; y++) {
        for (int x = 0; x < labels.cols; x++) {
            colors[labels.at<int>(y, x)] += img.at<cv::Vec3b>(y, x);
            counts[labels.at<int>(y, x)] ++;
        }
    }
    std::vector<cv::Vec3b> avgcolors(label_count, 0);
    for (int i = 0; i < label_count; i++) avgcolors[i] = colors[i] / counts[i];
    cv::Mat result_labels(labels.rows, labels.cols, CV_8UC3, cv::Scalar(0, 0, 0));
    for (int y = 0; y < labels.rows; y++) {
        for (int x = 0; x < labels.cols; x++) {
            result_labels.at<cv::Vec3b>(y, x) = avgcolors[labels.at<int>(y, x)];
        }
    }

    cv::imshow("labels", result_labels);
    std::cout << "press any key in a window ..." << std::endl;
    cv::waitKey(0);
    cv::destroyAllWindows();

    return 0;
}
