#include <opencv2/opencv.hpp>

cv::Mat grabcut(cv::Mat img, cv::Rect rect) {
    static cv::Mat mask(img.rows, img.cols, CV_8UC1, cv::Scalar(0));
    static uchar value = 0;
    static bool pressed = false;
    cv::MouseCallback mouseHandler2 = [](int event, int x, int y, int flags, void* userdata) {
        if (event == cv::EVENT_LBUTTONDOWN) {
            pressed = true;
        }
        else if (event == cv::EVENT_LBUTTONUP) {
            pressed = false;
        }
        if (pressed) {
            //mask.at<uchar>(y, x) = value;
            cv::circle(mask, cv::Point(x, y), 3, value, cv::FILLED);
        }
    };

    cv::namedWindow("grabcut");
    cv::setMouseCallback("grabcut", mouseHandler2);
    cv::namedWindow("output");
    cv::setMouseCallback("output", mouseHandler2);

    //printf("press B for Background, b for probable background,\n");
    //printf("F for Foregroud and f for probable foreground\n");
    //printf("then draw and press n for the next iteration\n");

    cv::Mat res, output;
    int iter = 0;
    int n = 1;
    cv::Mat bgdmodel(1, 65, CV_64FC1, cv::Scalar(0.0));
    cv::Mat fgdmodel(1, 65, CV_64FC1, cv::Scalar(0.0));
    for (;;) {
        if (n > iter) {
            if (iter == 0) { // grabcut with rect
                cv::grabCut(img, mask, rect, bgdmodel, fgdmodel, 1, cv::GC_INIT_WITH_RECT);
            }
            else { // grabcut with mask
                cv::grabCut(img, mask, rect, bgdmodel, fgdmodel, 1, cv::GC_INIT_WITH_MASK);
            }
            iter++;
        }

        output = img.clone();
        output.setTo(cv::Scalar(0,0,0), (mask == 0) | (mask == 2));
        cv::imshow("output", output);

        res = img.clone();
        res.setTo(cv::Scalar(0, 255, 0), (mask == 0));
        res.setTo(cv::Scalar(0, 0, 255), (mask == 1));
        res.setTo(cv::Scalar(255, 255, 0), (mask == 2));
        res.setTo(cv::Scalar(255, 0, 255), (mask == 3));
        cv::imshow("grabcut", res);
        int key = cv::waitKey(10);

        if (key == 'B') {
            value = 0;
            printf("select Background\n");
        }
        else if (key == 'F') {
            value = 1;
            printf("select Foreground\n");
        }
        else if (key == 'b') {
            value = 2;
            printf("select probable background\n");
        }
        else if (key == 'f') {
            value = 3;
            printf("select probable foreground\n");
        }
        else if (key == 'n') {
            n++;
            printf("next iteration, you may press Esc to escape\n");
        }
        else if (key == 27) break;
    }

    cv::rectangle(img, rect, (0, 255, 0), 2);
    cv::imwrite("grabcut-rect.png", img);
    cv::imwrite("grabcut-mask.png", res);
    cv::imwrite("grabcut-result.png", output);
    cv::destroyWindow("output");
    cv::destroyWindow("grabcut");
    return output;
}

int main(int argc, char ** argv) {

    // selection of the target 
    static cv::Point refPt0, refPt1;
    static cv::Point selPt0, selPt1;
    static enum { STARTING, SELECTING, SELECTED } selection = STARTING;
    cv::MouseCallback mouseHandler = [] (int event, int x, int y, int flags, void* userdata) {
        if (event == cv::EVENT_LBUTTONDOWN) {
            refPt0 = cv::Point(x, y);
            selection = SELECTING;
        }
        else if (event == cv::EVENT_LBUTTONUP) {
            selPt0 = refPt0;
            selPt1 = cv::Point(x, y);
            selection = SELECTED;
        }
        else if (selection == SELECTING) {
            refPt1 = cv::Point(x, y);
        }
    };
            
    cv::namedWindow("selection");
    cv::setMouseCallback("selection", mouseHandler);

    cv::Mat frame = cv::imread("../Face.jpg", cv::IMREAD_COLOR);

    printf("select rectangle with an object or press k to escape\n");
    cv::Rect rect;
    for (;;) {
        cv::Mat res = frame.clone() /*, target */;
        if (selection == SELECTED) {
            cv::Point tl(std::min(selPt0.x, selPt1.x), std::min(selPt0.y, selPt1.y));
            cv::Point br(std::max(selPt0.x, selPt1.x), std::max(selPt0.y, selPt1.y));
            //printf("[%d,%d] - [%d,%d]\n", tl.x, tl.y, br.x, br.y);
            if (tl.x < br.x && tl.y < br.y) {
                rect = cv::Rect(tl, br);
                //target = frame(rect);
            }
            selection = STARTING;
        }
        else if (selection == SELECTING) {
            cv::rectangle(res, cv::Point(std::min(refPt0.x, refPt1.x), std::min(refPt0.y, refPt1.y)), cv::Point(std::max(refPt0.x, refPt1.x), std::max(refPt0.y, refPt1.y)), cv::Scalar(0, 255, 0), 1);
        }
        else if (selection == SELECTED) {
            cv::rectangle(res, cv::Point(std::min(selPt0.x, selPt1.x), std::min(selPt0.y, selPt1.y)), cv::Point(std::max(selPt0.x, selPt1.x), std::max(selPt0.y, selPt1.y)), cv::Scalar(0, 255, 127), 1);
        }

        if (rect.area() > 0) {
            //printf("[%d,%d %dx%d]\n", rect.x, rect.y, rect.width, rect.height);
            grabcut(frame.clone(), rect);
            printf("select rectangle with an object or press k to escape\n");
            rect = cv::Rect();
        }
        cv::imshow("selection", res);
        int key = cv::waitKey(10);
        if (key == 'k') break;
    }

    cv::destroyAllWindows();
    return 0;
}



