import numpy as np
import cv2

def loga(value,base):
    return np.log(value)/np.log(base)
    
def singleScaleRetinex(img, sigma, a):
    retinex = loga(img,a) - loga(cv2.GaussianBlur(img, (0, 0), sigma),a)
    return retinex    

def MSRCR(src, sigma, low_perc, high_perc, N, a):
    img = 1.0 + src.astype(np.float64)
    img_retinex = singleScaleRetinex(img, sigma, a)
    levels = np.copy(img_retinex.reshape(-1))
    levels -= np.min(levels)
    levels /= np.max(levels)
    levels *= 255.0 / N
    levels = np.asarray(N*levels,np.int32)
    levels = np.sort(levels)
    zero_count = 0
    counts = np.zeros_like(levels)
    j = 0
    for i in range(len(levels)):
        if levels[i] == 0:
            zero_count += 1
        counts[j] += 1
        if i + 1 < len(levels) and levels[i] != levels[i + 1]:
            j += 1
    
    lev = np.copy(levels)
    levels, _ = np.unique(levels, return_counts=True)
    counts = counts[:len(levels)]
    low_val = levels[0] / N
    high_val = levels[-1] / N
    print('low_val',low_val,'high_val',high_val)
    for j in range(len(levels)):
        u = levels[j]
        c = counts[j]
        if u < 0 and c < zero_count*low_perc:
            low_val = u / N
        if u > 0 and c < zero_count*high_perc:
            high_val = u / N
            break
    img_retinex = np.maximum(np.minimum(img_retinex, high_val), low_val)

    minimum = np.min(img_retinex)
    maximum = np.max(img_retinex)
    img_retinex = 255 * (img_retinex - minimum) / (maximum - minimum)
    return img_retinex.astype(np.uint8)

if __name__ == '__main__':

    img = cv2.imread("../DinningRoom.jpg", cv2.IMREAD_GRAYSCALE)

    sigma = 29
    low_clip = 0.01
    high_clip = 0.99
    N = 160
    a = 10.0
    img_msrcr = MSRCR(img, sigma, low_clip, high_clip, N, a)

    cv2.imwrite("comics-DinningRoom.jpg", img_msrcr)
